;;; plus-word.el --- completing Ispell dictionary

(require 'ispell)

(defun +word (&optional quietly continue)
  "Check spelling of word under or before the cursor.
If the word is not found in dictionary, display possible corrections
in a window allowing you to choose one.

When the optional argument QUIETLY is non-nil or `ispell-quietly' is non-nil
when called interactively, non-corrective messages are suppressed.

With a prefix argument (or if CONTINUE is non-nil),
resume interrupted spell-checking of a buffer or region.

Word syntax described by `ispell-dictionary-alist' (which see).

This will check or reload the dictionary.  Use \\[ispell-change-dictionary]
or \\[ispell-region] to update the Ispell process.
"
  (interactive (list nil current-prefix-arg))
  (if continue
      (ispell-continue)
    (if (interactive-p)
	(setq following ispell-following-word
	      quietly ispell-quietly))
    (ispell-accept-buffer-local-defs)	; use the correct dictionary
    (let ((cursor-location (point))	; retain cursor location
	  (word (ispell-get-word t))
	  start end poss new-word)
      ;; De-structure return word info list.
      (setq start (car (cdr word))
	    end (car (cdr (cdr word)))
	    word (car word))

      ;; now check spelling of word if it has 3 or more characters.
      (cond
       ((> (length word) 2)
	(ispell-send-string "%\n")	; put in verbose mode
	(ispell-send-string (concat "^" word "\n"))
	;; wait until ispell has processed word
	 (while (progn
		  (ispell-accept-output)
		  (not (string= "" (car ispell-filter)))))
	;;(ispell-send-string "!\n") ;back to terse mode.
	(setq ispell-filter (cdr ispell-filter)) ; remove extra \n
	(if (and ispell-filter (listp ispell-filter))
	    (if (> (length ispell-filter) 1)
		(error "Ispell and its process have different character maps.")
	      (setq poss (ispell-parse-output (car ispell-filter)))))
	(cond ((eq poss t)
	       (or quietly
		   (message "%s jam estas konata"
			    (funcall ispell-format-word word))))
	      ((stringp poss)
	       (or quietly
		   (message "%s deriveblas el %s"
			    (funcall ispell-format-word word)
			    (funcall ispell-format-word poss))))
	      ((null poss) (message "Error in ispell process"))
	      
	      (t			; prompt for correct word.
	       (let ((guess (car (cdr (cdr (cdr poss)))))
		     (regulesprimo "\\<"))
;                   (print guess)
;                 (find-file "~/opt-pok/eo/iloj/ispell-nova/eo/vortoj.l3")
                 (find-file "~/lingvoj/ispell/ispell-eo/src/vortoj.l3")
		 (cond
                  (guess
                   (while (let ((string-i (car guess)))
                            (string-match
                             "\\([^+-]+\\+\\)?\\(\\w\\w+\\)\\(-\\w+\\)?\\+[^+]+$"
                             string-i)
                            (setq regulesprimo
                                  (concat regulesprimo
                                          (match-string 2 string-i)
                                          "\\>"))
                            (setq guess (cdr guess)))
                     (setq regulesprimo (concat regulesprimo "\\|\\<")))
;                       (goto-char (point-min))
                   (occur regulesprimo)
                   (switch-to-buffer "*Occur*"))
                  ((null guess)
                     (let ((regulesprimo (concat "\\<" word))
                           (n (+ 2 (length word))))
                       (kill-new (format "=\t%s/\n" word))
                       (while
                           (progn
                             (goto-char (point-min))
                             (and
                              (> (setq n (1- n)) 3)
                              (null (re-search-forward
                                     (substring regulesprimo 0 n)
                                     nil t)))))
                       (beginning-of-line))))))))))))

(global-set-key [f1] '+word)

(provide 'plus-word.el)